import json
from http import client
from http.client import HTTPResponse
from typing import List

from sk_core.security.security import generate_jwt_token, RequesterType, Requester


class Client:
    def __init__(self, host: str, port: int = 80, timeout: int = 5, scopes: List[str] = None) -> None:
        super().__init__()
        self.host = host
        self.port = port
        self.timeout = timeout
        self.scopes = scopes or []

    def _get_headers(self, requester: Requester):
        jwt = generate_jwt_token(
            requester.requester_id,
            RequesterType.service,
            scopes=self.scopes,
            company_id=requester.company_id
        )
        return {
            'Content-type': 'application/json',
            'Authorization': 'Bearer ' + jwt
        }

    def _handle_response(self, connection) -> dict:
        response: HTTPResponse = connection.getresponse()
        data = json.loads(response.read().decode())
        if response.status < 200 or response.status > 299:
            raise client.HTTPException(response.status, data)
        return data

    def get(self, requester: Requester, url: str, data: dict):
        headers = self._get_headers(requester=requester)
        connection = client.HTTPConnection(host=self.host, port=self.port, timeout=self.timeout)
        connection.request(method="GET", url=url, headers=headers, body=json.dumps(data))
        return self._handle_response(connection)
